# -*- coding: utf-8 -*-
# Licensed to Anthony Shaw (anthonyshaw@apache.org) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import requests

from pluralsight.exceptions import PluralsightApiException
from .invites import InvitesClient
from .users import UsersClient
from .teams import TeamsClient

BASE_URL = "https://app.pluralsight.com/plans/api/license/v1/{0}"


class LicensingAPIClient(object):
    """
    The licensing API client
    """

    def __init__(self, plan, api_key):
        """
        Instantiate a new reports API client

        :param plan: The plan name
        :type  plan: ``str``

        :param api_key: The API token (from the pluralsight team)
        :type  api_key: ``str``
        """
        self._plan = plan
        self._api_key = api_key

        self.base_url = BASE_URL.format(plan)

        self.session = requests.Session()
        self.session.headers.update(
            {"Accept": "application/json", "Authorization": "Token {0}".format(api_key)}
        )

        self.invites = InvitesClient(self)
        self.users = UsersClient(self)
        self.teams = TeamsClient(self)

    def get(self, uri, params=None):
        try:
            result = self.session.get(
                "{0}/{1}".format(self.base_url, uri), params=params
            )
            result.raise_for_status()

            return result.json()
        except requests.HTTPError as e:
            raise PluralsightApiException(e.response.text, uri)

    def post(self, uri, data=None):
        try:
            result = self.session.post("{0}/{1}".format(self.base_url, uri), json=data)
            result.raise_for_status()

            return result.json()
        except requests.HTTPError as e:
            raise PluralsightApiException(e.response.text)

    def put(self, uri, data=None):
        try:
            result = self.session.put("{0}/{1}".format(self.base_url, uri), json=data)
            result.raise_for_status()
        except requests.HTTPError as e:
            raise PluralsightApiException(e.response.text)

    def delete(self, uri):
        try:
            result = self.session.delete("{0}/{1}".format(self.base_url, uri))
            result.raise_for_status()
        except requests.HTTPError as e:
            raise PluralsightApiException(e.response.text)



# -*- coding: utf-8 -*-
# Licensed to Anthony Shaw (anthonyshaw@apache.org) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from requests_staticmock import (Adapter,
                                 BaseMockClass,
                                 mock_session_with_class)
import json
from pluralsight.licensing.client import LicensingAPIClient

TEST_PLAN = 'test_invites'
TEST_API_KEY = 'api_key'
TEST_URL = 'https://app.pluralsight.com'


def test_get_all_invites():
    """
    Test that get all invites returns properly formatted models
    """
    client = LicensingAPIClient(TEST_PLAN, TEST_API_KEY)
    special_adapter = Adapter('tests/fixtures')
    client.session.mount('https://app.pluralsight.com', special_adapter)
    invites = client.invites.get_all_invites()
    assert len(invites) == 2
    assert invites[0].id == "bc30c000-eeee-11e6-8088-111111111111"
    assert invites[0].email == 'a.guy@test.com'
    assert invites[0].team_id is None
    assert invites[0].note == 'Services'
    assert invites[0].send_date.timestamp == 1483488000
    assert invites[0].expires_on.timestamp == 1487376000
    assert str(invites[0]) == "Invite to a.guy@test.com (Services) " \
        "with ID: bc30c000-eeee-11e6-8088-111111111111"
    assert 'https://app.pluralsight.com/' in invites[0].generate_url('test')


def test_get_invites_filter():
    """
    Test that get invites with filter does apply the right params
    """
    _test_email = 'test.email@domain.com'
    _test_note = 'test note'
    _test_team = 'team 1'

    class TestMockClient(BaseMockClass):
        def _plans_api_license_v1_test_invites_invites(self, params, headers):
            if params['email'] == _test_email and \
                params['note'] == _test_note and \
                    params['teamId'] == _test_team:
                return json.dumps({'data': [{
                    "id": "bc30c000-dddd-11e6-80c5-46f6aaaaaaaa",
                    "email": _test_email,
                    "teamId": None,
                    "note": "Professional Services",
                    "sendDate": "2017-01-04T00:00:00+00:00",
                    "expiresOn": "2017-02-18T00:00:00+00:00"
                  }]})

    client = LicensingAPIClient(TEST_PLAN, TEST_API_KEY)

    with mock_session_with_class(client.session, TestMockClient,
                                 'https://app.pluralsight.com'):
        invites = client.invites.get_invites(email=_test_email,
                                             note=_test_note,
                                             team_id=_test_team)
        assert len(invites) == 1


def test_get_invite():
    """
    Test that get all invites returns properly formatted models
    """
    client = LicensingAPIClient('test_invite', TEST_API_KEY)

    special_adapter = Adapter('tests/fixtures')
    client.session.mount('https://app.pluralsight.com', special_adapter)
    invite = client.invites.get_invite("bc30c000-eeee-11e6-8088-111111111111")
    assert invite.id == "bc30c000-eeee-11e6-8088-111111111111"
    assert invite.email == 'a.guy@test.com'
    assert invite.team_id is None
    assert invite.note == 'Services'
    assert invite.send_date.timestamp == 1483488000
    assert invite.expires_on.timestamp == 1487376000


def test_create_invite():
    """
    Test that invites can be issued
    """
    client = LicensingAPIClient('test_create_invite', TEST_API_KEY)
    special_adapter = Adapter('tests/fixtures')
    client.session.mount('https://app.pluralsight.com', special_adapter)

    invite = client.invites.invite_user('test@test.com', None, "test note")
    assert invite.id == "bc30c000-eeee-11e6-8088-111111111111"
    assert invite.email == 'a.guy@test.com'
    assert invite.team_id is None
    assert invite.note == 'Services'
    assert invite.send_date.timestamp == 1483488000
    assert invite.expires_on.timestamp == 1487376000


def test_update_invite():
    """
    Test that invites can be updated
    """
    client = LicensingAPIClient('test_invite', TEST_API_KEY)
    special_adapter = Adapter('tests/fixtures')
    client.session.mount('https://app.pluralsight.com', special_adapter)

    client.invites.update_invite("bc30c000-eeee-11e6-8088-111111111111",
                                 "test note")
    assert True

def test_cancel_invite():
    """
    Test that invites can be updated
    """
    client = LicensingAPIClient('test_invite', TEST_API_KEY)
    special_adapter = Adapter('tests/fixtures')
    client.session.mount('https://app.pluralsight.com', special_adapter)
    client.invites.cancel_invite("bc30c000-eeee-11e6-8088-111111111111")



# -*- coding: utf-8 -*-
# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import requests

from pluralsight.exceptions import PluralsightApiException

BASE_URL = "https://app.pluralsight.com/plans/api/reports/v1/"


class ReportsAPIClient(object):
    """
    Reports API client
    """

    def __init__(self, plan, api_key):
        """
        Instantiate a new reports API client

        :param plan: The plan name
        :type  plan: ``str``

        :param api_key: The API token (from the pluralsight team)
        :type  api_key: ``str``
        """
        self._plan = plan
        self._api_key = api_key

        self.base_url = BASE_URL

        self.session = requests.Session()
        self.session.headers.update(
            {"Accept": "application/json", "Authorization": "Token {0}".format(api_key)}
        )

    def download_user_report(self, plan, path):
        """
        Download the user report and store in a file

        :param plan: The plan name
        :type  plan: ``str``

        :param path: Path to the downloaded CSV
        :type  path: ``str``

        :returns: The filename
        """
        return self._download_file("users/{0}".format(plan), "users.csv", path)

    def download_course_completion_report(
        self, plan, path, start_date=None, end_date=None
    ):
        """
        Download the course completion report and store in a file

        :param plan: The plan name
        :type  plan: ``str``

        :param path: Path to the downloaded CSV
        :type  path: ``str``

        :param start_date: (optional) Start date in format YYYY-MM-DD
        :type  start_date: ``str``

        :param end_date: (optional) End date in format YYYY-MM-DD
        :type  end_date: ``str``

        :returns: The filename
        """
        params = {}

        if start_date is not None:
            params["startDate"] = start_date
        if end_date is not None:
            params["endDate"] = end_date

        return self._download_file(
            "course-completion/{0}".format(plan), "courses.csv", path, params
        )

    def download_course_usage_report(self, plan, path, start_date=None, end_date=None):
        """
        Download the course usage report and store in a file

        :param plan: The plan name
        :type  plan: ``str``

        :param path: Path to the downloaded CSV
        :type  path: ``str``

        :param start_date: (optional) Start date in format YYYY-MM-DD
        :type  start_date: ``str``

        :param end_date: (optional) End date in format YYYY-MM-DD
        :type  end_date: ``str``

        :returns: The filename
        """
        params = {}

        if start_date is not None:
            params["startDate"] = start_date
        if end_date is not None:
            params["endDate"] = end_date

        return self._download_file(
            "course-usage/{0}".format(plan), "course_usage.csv", path, params
        )

    def _download_file(self, url, local_filename, path, params=None):
        try:
            r = self.session.get(
                "{0}{1}".format(self.base_url, url), params=params
            )
            r.encoding = 'utf-8-sig'
            r.raise_for_status()
            with open(os.path.join(path, local_filename), "w", encoding='utf-8') as f:
                f.write(r.text)
            return local_filename
        except requests.HTTPError as e:
            raise PluralsightApiException(e)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Test licensing client base functionality
"""
from requests_staticmock import (BaseMockClass,
                                 mock_session_with_class)
from requests_staticmock.responses import StaticResponseFactory
import json
import pytest
from six import b
from pluralsight.licensing.client import LicensingAPIClient
from pluralsight.exceptions import PluralsightApiException

TEST_PLAN = 'plana'
TEST_API_KEY = 'my_key_123'
TEST_URL = 'https://app.pluralsight.com'


class TestMockClient(BaseMockClass):
    def _plans_api_license_v1_plana_headers(self, request, headers):
        if 'Authorization' not in headers.keys() or \
                TEST_API_KEY not in headers['Authorization']:
            return json.dumps({'good': False})
        else:
            return json.dumps({'good': True})

    def _plans_api_license_v1_plana_get(self, method):
        return json.dumps({'good': method == 'GET'})

    def _plans_api_license_v1_plana_get_bad(self, request, method):
        if method == 'GET':
            return StaticResponseFactory.BadResponse(
                request=request,
                body=b('bad request'),
                status_code=500)

    def _plans_api_license_v1_plana_post(self, method):
        return json.dumps({'good': method == 'POST'})

    def _plans_api_license_v1_plana_post_bad(self, request, method):
        if method == 'POST':
            return StaticResponseFactory.BadResponse(
                request=request,
                body=b('bad request'),
                status_code=500)

    def _plans_api_license_v1_plana_put(self, method):
        return json.dumps({'good': method == 'PUT'})

    def _plans_api_license_v1_plana_put_bad(self, request, method):
        if method == 'PUT':
            return StaticResponseFactory.BadResponse(
                request=request,
                body=b('bad request'),
                status_code=500)

    def _plans_api_license_v1_plana_delete(self, method):
        return json.dumps({'good': method == 'DELETE'})

    def _plans_api_license_v1_plana_delete_bad(self, request, method):
        if method == 'DELETE':
            return StaticResponseFactory.BadResponse(
                request=request,
                body=b('bad request'),
                status_code=500)


client = LicensingAPIClient(TEST_PLAN, TEST_API_KEY)


def test_base_url():
    assert TEST_PLAN in client.base_url


def test_headers():
    with mock_session_with_class(client.session, TestMockClient, TEST_URL):
        response = client.get('headers')
        assert response['good']


def test_get_request():
    with mock_session_with_class(client.session, TestMockClient, TEST_URL):
        response = client.get('get')
        assert response['good']


def test_get_bad_request():
    with mock_session_with_class(client.session, TestMockClient, TEST_URL):
        with pytest.raises(PluralsightApiException):
            client.get('get/bad')


def test_post_request():
    with mock_session_with_class(client.session, TestMockClient, TEST_URL):
        response = client.post('post')
        assert response['good']


def test_post_bad_request():
    with mock_session_with_class(client.session, TestMockClient, TEST_URL):
        with pytest.raises(PluralsightApiException):
            client.post('post/bad')


def test_put_request():
    with mock_session_with_class(client.session, TestMockClient, TEST_URL):
        response = client.put('put')
        assert response is None


def test_put_bad_request():
    with mock_session_with_class(client.session, TestMockClient, TEST_URL):
        with pytest.raises(PluralsightApiException):
            client.put('put/bad')


def test_delete_request():
    with mock_session_with_class(client.session, TestMockClient, TEST_URL):
        client.delete('delete')
        assert True


def test_delete_bad_request():
    with mock_session_with_class(client.session, TestMockClient, TEST_URL):
        with pytest.raises(PluralsightApiException):
            client.delete('delete/bad')



# -*- coding: utf-8 -*-
# Licensed to Anthony Shaw (anthonyshaw@apache.org) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from requests_staticmock import (Adapter,
                                 BaseMockClass,
                                 mock_session_with_class)
from requests_staticmock.responses import StaticResponseFactory
from six import b
import csv
import pytest
from pluralsight.exceptions import PluralsightApiException
from pluralsight.reports.client import ReportsAPIClient, BASE_URL

TEST_PLAN = 'test-plan'
TEST_API_KEY = 'api_key'
TEST_URL = 'https://app.pluralsight.com'


client = ReportsAPIClient(TEST_PLAN, TEST_API_KEY)
special_adapter = Adapter('tests/fixtures')
client.session.mount('https://app.pluralsight.com', special_adapter)


def test_base_url():
    assert BASE_URL in client.base_url


def test_download_user_report():
    report_name = client.download_user_report(TEST_PLAN, '')
    with open(report_name, 'r') as f:
        reader = csv.DictReader(f)
        data = [d for d in reader]
        assert data[0]['UserId'] == '12919ae2-c23f-480a-96a8-1534293b0bbe'
        assert data[0]['FirstName'] == 'Sandeep'
        assert data[1]['FirstName'] == 'Imranullah'


def test_download_course_completion_report():
    report_name = client.download_course_completion_report(TEST_PLAN, '',
                                                           start_date='2016-04-02',
                                                           end_date='2016-05-02')
    with open(report_name, 'r') as f:
        reader = csv.DictReader(f)
        data = [d for d in reader]
        assert data[0]['FirstName'] == 'Jason'
        assert data[1]['FirstName'] == 'Alex'


def test_download_course_usage_report():
    report_name = client.download_course_usage_report(TEST_PLAN, '',
                                                      start_date='2016-04-02',
                                                      end_date='2016-05-02')
    with open(report_name, 'r') as f:
        reader = csv.DictReader(f)
        data = [d for d in reader]
        assert data[0]['FirstName'] == 'Sam'
        assert data[1]['FirstName'] == 'Alexey'


def test_get_bad_request():
    class TestMockClient(BaseMockClass):
        def _plans_api_reports_v1_users_test_plan(self, request, method):
            return StaticResponseFactory.BadResponse(
                request=request,
                body=b('bad request'),
                status_code=500)
    with mock_session_with_class(client.session, TestMockClient, TEST_URL):
        with pytest.raises(PluralsightApiException):
            client.download_user_report(TEST_PLAN, '')



# -*- coding: utf-8 -*-
# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class User(object):
    def __init__(self, id, team_id, first_name, last_name, email, note, start_date):
        self.id = id
        self.team_id = team_id
        self.first_name = first_name
        self.last_name = last_name
        self.email = email
        self.note = note
        self.start_date = start_date

    def __str__(self):
        return "User '{0} {1}' {2} ({3})".format(
            self.first_name, self.last_name, self.email, self.id
        )



# -*- coding: utf-8 -*-
# Licensed to Anthony Shaw (anthonyshaw@apache.org) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from requests_staticmock import (Adapter,
                                 BaseMockClass,
                                 mock_session_with_class)
import json
from pluralsight.licensing.client import LicensingAPIClient

TEST_PLAN = 'test_teams'
TEST_API_KEY = 'api_key'
TEST_URL = 'https://app.pluralsight.com'


def test_get_all_teams():
    """
    Test that get all invites returns properly formatted models
    """
    client = LicensingAPIClient(TEST_PLAN, TEST_API_KEY)
    special_adapter = Adapter('tests/fixtures')
    client.session.mount('https://app.pluralsight.com', special_adapter)
    teams = client.teams.get_all_teams()
    assert len(teams) == 2
    assert teams[0].id == '2b947975-482a-4791-aa40-e199b3ca8738'
    assert teams[0].name == 'FED'
    assert str(teams[0]) == "Team 'FED' (2b947975-482a-4791-aa40-e199b3ca8738)"


def test_get_teams_filter():
    """
    Test that get invites with filter does apply the right params
    """

    class TestMockClient(BaseMockClass):
        def _plans_api_license_v1_test_teams_teams(self, params, headers):
            if params['name'] == 'test':
                return json.dumps({'data': [{
                    "id": '2b947975-482a-4791-aa40-e199b3ca8738',
                    "name": 'test'
                  }]})

    client = LicensingAPIClient(TEST_PLAN, TEST_API_KEY)

    with mock_session_with_class(client.session, TestMockClient,
                                 'https://app.pluralsight.com'):
        teams = client.teams.get_all_teams(name='test')
        assert len(teams) == 1
        assert teams[0].name == 'test'


def test_get_team():
    """
    Test that you can fetch a single team
    """
    client = LicensingAPIClient('test_team', TEST_API_KEY)

    special_adapter = Adapter('tests/fixtures')
    client.session.mount('https://app.pluralsight.com', special_adapter)
    team = client.teams.get_team("2b947975-482a-4791-aa40-e199b3ca8738")
    assert team.id == "2b947975-482a-4791-aa40-e199b3ca8738"
    assert team.name == 'FED'



# -*- coding: utf-8 -*-
# Licensed to Anthony Shaw (anthonyshaw@apache.org) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pluralsight.models.team import Team


class TeamsClient(object):
    """
    Teams API
    """

    def __init__(self, client):
        self.client = client

    def get_all_teams(self, name=None):
        """
        Get all teams

        :param name: Filter by name
        :type  name: ``str``

        :return: A list of :class:`Team`
        :rtype: ``list`` of :class:`Team`
        """
        params = {}

        if name is not None:
            params["name"] = name

        teams = self.client.get("teams", params=params)
        return [self._to_team(i) for i in teams["data"]]

    def get_team(self, id):
        """
        Fetch a team by ID

        :param id: The identifier
        :type  id: ``str``

        :return: An instance :class:`Team`
        :rtype: :class:`Team`
        """
        team = self.client.get("teams/{0}".format(id))
        return self._to_team(team["data"])

    def _to_team(self, data):
        return Team(data["id"], data["name"])



# -*- coding: utf-8 -*-
# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import six


class Invite(object):
    def __init__(self, id, email, team_id, note, send_date, expires_on):
        self.id = id
        self.email = email
        self.team_id = team_id
        self.note = note
        self.send_date = send_date
        self.expires_on = expires_on

    def __str__(self):
        return "Invite to {0} ({1}) with ID: {2}".format(self.email, self.note, self.id)

    def generate_url(self, plan):
        _redirect_url = "https://app.pluralsight.com/plans-data/invites/{0}/{1}".format(
            plan, self.id
        )
        _base_url = (
            "https://app.pluralsight.com/id/createaccount/business"
            "?firstName={0}&lastName={1}&companyEmail={2}&redirectTo={3}".format(
                "",
                "",
                six.moves.urllib.parse.quote(self.email),
                six.moves.urllib.parse.quote_plus(_redirect_url),
            )
        )
        return _base_url



# -*- coding: utf-8 -*-
# Licensed to Anthony Shaw (anthonyshaw@apache.org) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from requests_staticmock import (Adapter,
                                 BaseMockClass,
                                 mock_session_with_class)
import json
from pluralsight.licensing.client import LicensingAPIClient

TEST_PLAN = 'test_users'
TEST_API_KEY = 'api_key'
TEST_URL = 'https://app.pluralsight.com'


def test_get_all_users():
    """
    Test that get all users returns properly formatted models
    """
    client = LicensingAPIClient(TEST_PLAN, TEST_API_KEY)
    special_adapter = Adapter('tests/fixtures')
    client.session.mount('https://app.pluralsight.com', special_adapter)
    users = client.users.get_all_users()
    assert len(users) == 2
    assert users[0].id == "0bbdccac-dad1-4488-a69d-2ea78bf43280"
    assert users[0].team_id == "2b947975-482a-4791-aa40-e199b3ca8738"
    assert users[0].first_name == "first-25"
    assert users[0].last_name == "last-25"
    assert users[0].email == "email-25+9040@email.com"
    assert users[0].note == 'test note'
    assert users[0].start_date.timestamp == 1463020267
    assert str(users[0]) == "User 'first-25 last-25' email-25+9040@email.com " \
        "(0bbdccac-dad1-4488-a69d-2ea78bf43280)"


def test_get_users_filter():
    """
    Test that get users with filter does apply the right params
    """
    _first_name = 'fred'
    _last_name = 'flintstone'
    _email = 'fred@flintstone.com'
    _note = 'yabba dabba doo'
    _team_id = '2b947975-482a-4791-aa40-e199b3ca8738'

    class TestMockClient(BaseMockClass):
        def _plans_api_license_v1_test_users_users(self, params, headers):
            assert params['firstName'] == _first_name
            assert params['lastName'] == _last_name
            assert params['email'] == _email
            assert params['note'] == _note
            assert params['teamId'] == _team_id

            return json.dumps({'data': [{
                    "id": "0bbdccac-dad1-4488-a69d-2ea78bf43280",
                    "teamId": "2b947975-482a-4791-aa40-e199b3ca8738",
                    "firstName": "first-25",
                    "lastName": "last-25",
                    "email": "email-25+9040@email.com",
                    "note": "test note",
                    "startDate": "2016-05-12T02:31:07.233+00:00"
                  }]})

    client = LicensingAPIClient(TEST_PLAN, TEST_API_KEY)

    with mock_session_with_class(client.session, TestMockClient,
                                 'https://app.pluralsight.com'):
        users = client.users.get_all_users(first_name=_first_name,
                                           last_name=_last_name,
                                           email=_email,
                                           note=_note,
                                           team_id=_team_id)
        assert len(users) == 1
        assert users[0].id == "0bbdccac-dad1-4488-a69d-2ea78bf43280"
        assert users[0].team_id == "2b947975-482a-4791-aa40-e199b3ca8738"
        assert users[0].first_name == "first-25"
        assert users[0].last_name == "last-25"
        assert users[0].email == "email-25+9040@email.com"
        assert users[0].note == 'test note'
        assert users[0].start_date.timestamp == 1463020267


def test_get_user():
    """
    Test that you can fetch a single user
    """
    client = LicensingAPIClient('test_user', TEST_API_KEY)

    special_adapter = Adapter('tests/fixtures')
    client.session.mount('https://app.pluralsight.com', special_adapter)
    user = client.users.get_user("3d4a29f7-aedc-46b8-bada-f84d04f98679")
    assert user.id == "3d4a29f7-aedc-46b8-bada-f84d04f98679"


def test_update_user():
    """
    Test that users can be updated
    """
    client = LicensingAPIClient('test_user', TEST_API_KEY)
    special_adapter = Adapter('tests/fixtures')
    client.session.mount('https://app.pluralsight.com', special_adapter)

    client.users.update_user(id="3d4a29f7-aedc-46b8-bada-f84d04f98679",
                             team_id="1234512-aedc-46b8-bada-f84d04f98679",
                             note="test note")
    assert True


def test_delete_user():
    """
    Test that users can be deleted
    """
    client = LicensingAPIClient('test_user', TEST_API_KEY)
    special_adapter = Adapter('tests/fixtures')
    client.session.mount('https://app.pluralsight.com', special_adapter)
    client.users.delete_user("3d4a29f7-aedc-46b8-bada-f84d04f98679")



# -*- coding: utf-8 -*-
# Licensed to Anthony Shaw (anthonyshaw@apache.org) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import arrow
from pluralsight.models.user import User


class UsersClient(object):
    """
    Users API
    """

    def __init__(self, client):
        self.client = client

    def get_all_users(
        self, first_name=None, last_name=None, email=None, note=None, team_id=None
    ):
        """
        Get all users

        :param first_name: Filter by first name
        :type  first_name: ``str``

        :param last_name: Filter by last name
        :type  last_name: ``str``

        :param email: Filter by email
        :type  email: ``str``

        :param note: Filter by note
        :type  note: ``str``

        :param team_id: Filter by team ID
        :type  team_id: ``str``

        :return: A list of :class:`User`
        :rtype: ``list`` of :class:`User`
        """
        params = {}

        if first_name is not None:
            params["firstName"] = first_name
        if last_name is not None:
            params["lastName"] = last_name
        if email is not None:
            params["email"] = email
        if note is not None:
            params["note"] = note
        if team_id is not None:
            params["teamId"] = team_id

        users = self.client.get("users", params=params)
        return [self._to_user(i) for i in users["data"]]

    def get_user(self, id):
        """
        Fetch a user by ID

        :param id: The identifier
        :type  id: ``str``

        :return: An instance :class:`User`
        :rtype: :class:`User`
        """
        user = self.client.get("users/{0}".format(id))
        return self._to_user(user["data"])

    def update_user(self, id, team_id, note):
        """
        Update a user

        :param id: The identifier
        :type  id: ``str``

        :param team_id: Team the user belongs to
        :type  team_id: ``str``

        :param note: Additional notes on the user
        :type  note: ``str``
        """
        data = {"data": {"teamId": team_id, "note": note}}
        self.client.put("users/{0}".format(id), data=data)

    def delete_user(self, id):
        """
        Delete an existing user

        :param id: The identifier
        :type  id: ``str``

        :rtype: None
        """
        self.client.delete("users/{0}".format(id))

    def _to_user(self, data):
        return User(
            data["id"],
            data["teamId"],
            data["firstName"],
            data["lastName"],
            data["email"],
            data["note"],
            arrow.get(data["startDate"]),
        )



# -*- coding: utf-8 -*-
# Licensed to Anthony Shaw (anthonyshaw@apache.org) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import arrow
from pluralsight.models.invite import Invite


class InvitesClient(object):
    """
    Invites API
    """

    def __init__(self, client):
        self.client = client

    def get_all_invites(self):
        """
        Get all invites

        :return: A list of :class:`Invite`
        :rtype: ``list`` of :class:`Invite`
        """
        return self.get_invites()

    def get_invites(self, email=None, note=None, team_id=None):
        """
        Get invitations matching certain filters

        :param email: The users' email address
        :type  email: ``str``

        :param team_id: The team identifier
        :type  team_id: ``str``

        :param note: Additional notes on the user
        :type  note: ``str``

        :return: A list of :class:`Invite`
        :rtype: ``list`` of :class:`Invite`
        """
        params = {}
        if email is not None:
            params["email"] = email
        if note is not None:
            params["note"] = note
        if team_id is not None:
            params["teamId"] = team_id
        invites = self.client.get("invites", params=params)
        return [self._to_invite(i) for i in invites["data"]]

    def get_invite(self, id):
        """
        Fetch an invitation by ID

        :param id: The identifier
        :type  id: ``str``

        :return: An instance :class:`Invite`
        :rtype: :class:`Invite`
        """
        invite = self.client.get("invites/{0}".format(id))
        return self._to_invite(invite["data"])

    def invite_user(self, email, team_id=None, note=None):
        """
        Create a new invitation

        :param email: The users' email address
        :type  email: ``str``

        :param team_id: The team identifier
        :type  team_id: ``str``

        :param note: Additional notes on the user
        :type  note: ``str``

        :return: An instance :class:`Invite`
        :rtype: :class:`Invite`
        """
        data = {"data": {"email": email, "teamId": team_id, "note": note}}
        invite = self.client.post("invites", data=data)
        return self._to_invite(invite["data"])

    def update_invite(self, id, note):
        """
        Update an invitation

        :param id: The identifier
        :type  id: ``str``

        :param note: Additional notes on the user
        :type  note: ``str``
        """
        data = {"data": {"note": note}}
        self.client.put("invites/{0}".format(id), data=data)

    def cancel_invite(self, id):
        """
        Cancel an invitation

        :param id: The identifier
        :type  id: ``str``

        :rtype: None
        """
        self.client.delete("invites/{0}".format(id))

    def _to_invite(self, data):
        return Invite(
            id=data["id"],
            email=data["email"],
            team_id=data["teamId"],
            note=data["note"],
            send_date=arrow.get(data["sendDate"]),
            expires_on=arrow.get(data["expiresOn"]),
        )
